@@ -19,7 +19,7 @@
  * this, these needs to be prototyped.  For gcc 2.5.x, these are either
  * built in, or defined in the fixed header files.
  */
-#if !defined(__GNUC_MINOR__) || (__GNUC_MINOR__ == 6)
+#if (!defined(__GNUC_MINOR__) || (__GNUC_MINOR__ >= 6)) && !defined(SOLARIS2)
 void    printf(char* , ...);
 void    puts(char *);
 void    fprintf(FILE*, char *, ...);
@@ -29,7 +29,14 @@ int sscanf(char *, ...);
 int     sleep(unsigned);
 int     atoi(char *);
 int	gethostname(char *, int);
+#if !defined(SOLARIS2)
 char 	*strcat(char *, char*);
 char	*strchr(char*, char);
 #endif
 #endif
+#endif
+
+#if defined(__pyrsoft)
+char *strchr(char *, char);
+#endif
+
