@@ -10,7 +10,7 @@ struct base_types {
 
 struct Command_Line_Options {
 	char	*cmd_option;	/* how it is called on the command line */
-	int	num_args:1;	/* Does it have an argument? */
+	char	num_args;	/* Number of options for this command */
 	short	pass;		/* What pass this should be processed on. */
 	void	(*func)();	/* function to call when we match this.
 				 * if num_args is true, than that gets passed
@@ -21,15 +21,31 @@ struct Command_Line_Options {
 /* It is important that the n'th placement of this array corresponds
  * to the n'th value type in xsysstats.h (ie, 9'th value corresponds to errors)
  */
-struct base_types types[NUM_TYPES+1] = {
-{"cpu", "cpu", 100}, {"packets", "pkts",32}, {"page", "page",16},
-{"swap", "swap",16},{"interrupts", "intr",200}, {"disk", "disk",40},
+struct base_types types[NUM_TYPES] = {
+{"cpu", "cpu", 100}, 
+{"packets", "pkts",32}, 
+{"pagei", "pagei",16},
+{"swapi", "iswap",16},
+{"interrupts", "intr",200}, 
+{"disk", "disk",40},
 {"context", "ctxt",128},
 {"load1", "load1",2},
 {"load5", "load5", 2},
 {"load15", "load15", 2},
 {"collisions", "coll",8},
-{"errors", "errs",8}
+{"errors", "errs",8},
+{"packetsi", "pktsi",32},
+{"packetso", "pktso",32},
+{"apagei", "apagei",16},
+{"apageo", "apageo",16},
+{"pageo", "pageo",16},
+{"page", "page",16},
+{"swapo", "swapo",16},
+{"swap", "swap",16},
+{"ucpu", "ucpu", 100}, 
+{"ncpu", "ncpu", 100}, 
+{"scpu", "scpu", 100}, 
+{"icpu", "icpu", 100}, 
 };
 
 
@@ -39,10 +55,15 @@ struct base_types types[NUM_TYPES+1] = {
  * it is rstat_value * vertical_size / scale.
  */
 
+typedef enum  {Nolabel, Minimal, Normal} LabelStyle;
+
+LabelStyle label_style=Normal;
+
 int	screen_num, num_graphs=0, **points,point_pos=-1,sleep_time=2,
 	font_height=0, num_info_lines,
 	draw_baseline=FALSE,allnames=FALSE,
-	num_hosts=0,localhost=-1,hide_labels=FALSE,show_time=FALSE,
+	num_hosts=0,localhost=-1,
+	show_time=FALSE,
 	time_ticks=45,		/* How many seconds a test entry to the
 				 * ruler should be printed.
 				 */
@@ -66,6 +87,10 @@ int	screen_num, num_graphs=0, **points,point_pos=-1,sleep_time=2,
 	scale_sync=FALSE,	/* True if all the graphs should have the
 				 * same scale.
 				 */
+	solid=FALSE,		/* True if drawing solid (not line) graph
+				 */
+        hide_hosts=FALSE,	/* If true, don't display hostnames. */
+	show_max=FALSE,		/* If true, show max value graph ever hit */
 	graph_spacing=INSIDE_BORDER_WIDTH;
 				/* How many pixels should be between each
 				 * graph and the edge of the window.
@@ -79,15 +104,16 @@ XFontStruct	*font_info;
 GC	gc;
 XColor	background, foreground,baseline;
 Pixmap	new_win;	/* used for scrolling */
-char	**hosts,
-	*display_name="";		/* name to open the display with */
+char	*display_name="";		/* name to open the display with */
 struct Xss_Window **windows=NULL;
-
+struct Host_Info *hosts;
 
 XSizeHints size_hints = { PMinSize | PWinGravity,0,0,DEFAULT_SIZE,
 	DEFAULT_SIZE,DEFAULT_SIZE, DEFAULT_SIZE, 0,0,0,0,{0,0},{0,0},0,0,
 	NorthWestGravity};
 
+static unsigned long defcolors[NUM_TYPES];
+char *window_name, *window_title=VERSION;
 
 char *strdup_local(char *str)
 {
@@ -98,12 +124,100 @@ char *strdup_local(char *str)
 	return s;
 }
 
+int minimal_label_lines(int windowno)
+{
+    int labelwidth=0, lines_needed=0,dummy,i;
+    char tmp[80];
+    XCharStruct	scale_info;
+
+    /* First is to see how many lines we need for the labels. Use a 3
+     *  pixel spacing between each element.
+     */
+    if (windows[windowno]->title!=NULL) {
+	XTextExtents(font_info, windows[windowno]->title, strlen(windows[windowno]->title), 
+		     &dummy, &dummy, &dummy, &scale_info);
+	labelwidth = scale_info.width+3;
+    }
+    if (labelwidth > windows[windowno]->width) {
+	labelwidth=0;
+	lines_needed++;
+    }
+    for (i=0; i<num_graphs; i++) {
+	if (graphs[i].window!=windowno) continue;
+	if (show_max)
+	    sprintf(tmp, "%d/%d", graphs[i].scale, graphs[i].max_val/graphs[i].scale_mult);
+	else
+	    sprintf(tmp,"%d", graphs[i].scale);
+	XTextExtents(font_info, tmp, strlen(tmp), 
+		     &dummy, &dummy, &dummy, &scale_info);
+	labelwidth += scale_info.width+3;
+	if (labelwidth > windows[windowno]->width) {
+	    labelwidth=scale_info.width+3;
+	    lines_needed++;
+	}
+    }
+    /* if there is leftover, it will have to go on the next line. */
+    if (labelwidth) lines_needed++;
+    return lines_needed;
+}
+
+/* Draws the window label when in Minimal mode. */
+void draw_minimal_label(int windowno)
+{
+    int labelwidth=0, on_line=0,dummy,i;
+    char tmp[80];
+    XCharStruct	scale_info;
+
+    /* 
+     * First, draw the window title.
+     */
+    if (windows[windowno]->title!=NULL) {
+	XTextExtents(font_info, windows[windowno]->title, strlen(windows[windowno]->title), 
+		     &dummy, &dummy, &dummy, &scale_info);
+	labelwidth = scale_info.width+3;
+	XSetForeground(display, gc, foreground.pixel);
+	XDrawImageString(display, win, gc,
+		windows[windowno]->x, 
+		windows[windowno]->y + windows[windowno]->height + 1 +
+		font_height*show_time + font_info->max_bounds.ascent,
+		windows[windowno]->title, strlen(windows[windowno]->title));
+    }
+    if (labelwidth > windows[windowno]->width) {
+	labelwidth=0;
+	on_line++;
+    }
+    /*
+     * Now go through and draw each graph.
+     */
+    for (i=0; i<num_graphs; i++) {
+	if (graphs[i].window!=windowno) continue;
+	if (show_max)
+	    sprintf(tmp, "%d/%d", graphs[i].scale, graphs[i].max_val/graphs[i].scale_mult);
+	else
+	    sprintf(tmp,"%d", graphs[i].scale);
+	XTextExtents(font_info, tmp, strlen(tmp), 
+		     &dummy, &dummy, &dummy, &scale_info);
+
+	if (labelwidth + scale_info.width+3> windows[windowno]->width) {
+	    on_line++;
+	    labelwidth=0;
+	}
+	XSetForeground(display, gc, graphs[i].color_pixel);
+	XDrawImageString(display, win, gc, 
+		windows[windowno]->x + labelwidth,
+	        windows[windowno]->y + windows[windowno]->height + 1 +
+	        font_height * (show_time+on_line) + 
+		font_info->max_bounds.ascent, tmp, strlen(tmp));
+	labelwidth += scale_info.width+3;
+    }
+}
+
 /* This function takes a window number as an argument, and re-draws that
  * window.
  */
 void redraw_graph(int windowno)
 {
-	int	i,pp,x,max_x,font_line=0,fontx=0,fonts_per_line,deltax,pp1;
+	int	i,pp,x,max_x,font_line=0,fontx=0,fonts_per_line,deltax,pp1,dummy;
 	char	tmp[256];
 	XCharStruct	scale_info;
 
@@ -120,13 +234,15 @@ void redraw_graph(int windowno)
 	}
 	if (show_time) {
 	    font_line = 1; /* so that the type/scale information is one
-			    * line further down, living room for the rule.
+			    * line further down, leaving room for the rule.
 			    */
 	    XSetForeground(display, gc, foreground.pixel);
 	    for (i=0; i*time_ticks < windows[windowno]->width * sleep_time; i++) {
-		int dummy;
 
-		sprintf(tmp, "%d", time_ticks * i);
+		if ((time_ticks * i )%60 == 0) 
+		    sprintf(tmp, "%dm", time_ticks * i / 60);
+		else
+		    sprintf(tmp, "%d", time_ticks * i);
 		XTextExtents(font_info, tmp, strlen(tmp), 
 			&dummy, &dummy, &dummy, &scale_info);
 
@@ -142,6 +258,7 @@ void redraw_graph(int windowno)
 		    windows[windowno]->y + windows[windowno]->height + 4);
 	    }
 	}
+	if (label_style==Minimal) draw_minimal_label(windowno);
 	fonts_per_line = windows[windowno]->width / TEXT_WIDTH;
 	deltax = windows[windowno]->width / fonts_per_line;
 
@@ -149,14 +266,17 @@ void redraw_graph(int windowno)
 	    if (graphs[i].window!=windowno) continue;
 
 	    XSetForeground(display, gc, graphs[i].color_pixel);
-	    if (!hide_labels) {
+	    if (label_style==Normal) {
 		XDrawImageString(display, win, gc,
 			windows[windowno]->x + fontx * deltax, 
 			windows[windowno]->y + windows[windowno]->height + 1 +
 			font_height*font_line + font_info->max_bounds.ascent,
 			graphs[i].name, graphs[i].name_len);
 
-		sprintf(tmp, "%d", graphs[i].scale);
+		if (show_max)
+		    sprintf(tmp, "%d/%d", graphs[i].scale, graphs[i].max_val/graphs[i].scale_mult);
+		else
+		    sprintf(tmp,"%d", graphs[i].scale);
 
 		XTextExtents(font_info, tmp, strlen(tmp), 
 			&x, &x, &x, &scale_info);
@@ -172,6 +292,7 @@ void redraw_graph(int windowno)
 			font_line++;
 		}
 	    }
+
 	    if ( windows[windowno]->xpos<windows[windowno]->width) {
 		pp=0;
 		max_x =  windows[windowno]->xpos-1;
@@ -182,13 +303,23 @@ void redraw_graph(int windowno)
 	    }
 	    for (x=0; x<max_x; x++) {
 		pp1 = (pp +1) % split_width;
-		XDrawLine(display,win, gc, x + windows[windowno]->x, 
+		if (solid) {
+                   XFillRectangle(display,win, gc, x + windows[windowno]->x,
+                        1+windows[windowno]->y + windows[windowno]->height -
+                        points[i][pp] * windows[windowno]->height/
+                            (graphs[i].scale * graphs[i].scale_mult),
+                        1,
+                        (points[i][pp] * windows[windowno]->height) /
+                        (graphs[i].scale * graphs[i].scale_mult));
+		} else {
+		  XDrawLine(display,win, gc, x + windows[windowno]->x, 
 		    windows[windowno]->y + windows[windowno]->height -
 		    points[i][pp] * windows[windowno]->height/(graphs[i].scale *
 		    graphs[i].scale_mult), x+windows[windowno]->x+1,
 		    windows[windowno]->y + windows[windowno]->height -
 		    (points[i][pp1] * windows[windowno]->height) /
 		    (graphs[i].scale * graphs[i].scale_mult));
+		}
 		pp = pp1;
 		/* point_pos %2 is to draw it every other point. */
 		if (graphs[i].scale_lines>0 && !(pp % 2) &&
@@ -205,7 +336,7 @@ void redraw_graph(int windowno)
 			 * the top of the window.
 			 */
 			while (sline*graphs[i].scale_lines < graphs[i].scale) {
-			    XDrawPoint(display, win, gc, x,
+			    XDrawPoint(display, win, gc, x + windows[windowno]->x,
 				windows[windowno]->y + windows[windowno]->height
 				- (sline * graphs[i].scale_lines * 
 				windows[windowno]->height) /
@@ -256,9 +387,13 @@ void set_graph_windows()
 		    graph_spacing*(ypos+1);
 
 		windows[window_num]->height = split_height;
-		if (!hide_labels)
+		if (label_style == Normal)
 		    windows[window_num]->height -= font_height *
 			windows[window_num]->label_lines;
+		else if (label_style == Minimal) {
+		    windows[window_num]->height -= 
+			font_height*minimal_label_lines(window_num);
+		}
 		if (show_time)
 		    windows[window_num]->height -= font_height;
 	    }
@@ -391,16 +526,32 @@ void process_keypress(char *keypress)
     switch (keypress[0]) {
 	case 'l':
 	case 'L':
-		hide_labels = !(hide_labels);
+		label_style++;
+		if (label_style > Normal) label_style=Nolabel;
 		set_graph_windows();
 		redraw_all_graphs();
 		break;
+	case 'm':
+	case 'M':
+		show_max = !(show_max);
+		redraw_all_graphs();
+		break;
 	case 'r':
 	case 'R':
 		show_time = !(show_time);
 		set_graph_windows();
 		redraw_all_graphs();
 		break;
+	case 12: /* C-l */
+		redraw_all_graphs();
+		break;
+
+#if defined(KEYQUIT)
+	case 'q':
+	case 'Q':
+	    XCloseDisplay(display);
+	    exit(0);
+#endif
     }
 }
 
@@ -423,6 +574,7 @@ void check_events()
 			break;
 
 		case KeyPress:
+			keystring[0]=0;
 			XLookupString((XKeyEvent *) &event, keystring,10, NULL, NULL);
 			process_keypress(keystring);
 			break;
@@ -461,6 +613,7 @@ void update_window(int windowno)
 	}
 	/* Need to scroll the window over */
 	else if (windows[windowno]->xpos>windows[windowno]->width) {
+#if 0
 	    XCopyArea(display, win, new_win, gc, windows[windowno]->x+1,
 		windows[windowno]->y, windows[windowno]->width, windows[windowno]->height+1,
 		0, 0);
@@ -470,6 +623,14 @@ void update_window(int windowno)
 	    XCopyArea(display, new_win, win, gc, 0, 0, 
 		windows[windowno]->width, windows[windowno]->height+1, 
 		windows[windowno]->x, windows[windowno]->y);
+#else
+	    XCopyArea(display, win, win, gc, windows[windowno]->x+1,
+		windows[windowno]->y, windows[windowno]->width, windows[windowno]->height+1,
+		windows[windowno]->x, windows[windowno]->y);
+	    XClearArea(display, win, windows[windowno]->x +
+		windows[windowno]->width , windows[windowno]->y,
+		1, windows[windowno]->height+1, False);
+#endif
 	    windows[windowno]->xpos--;
 	}
         if (draw_baseline) {
@@ -483,15 +644,26 @@ void update_window(int windowno)
 
 	    XSetForeground(display, gc, graphs[count].color_pixel);
 
-            XDrawLine(display,win, gc,
-		windows[windowno]->x + windows[windowno]->xpos-1,
-		windows[windowno]->y + windows[windowno]->height-
-		lasty[count] * windows[windowno]->height/(graphs[count].scale *
-		graphs[count].scale_mult),
-		windows[windowno]->x + windows[windowno]->xpos,
-		windows[windowno]->y +windows[windowno]->height - 
-		(points[count][point_pos] *windows[windowno]->height) /
-		(graphs[count].scale * graphs[count].scale_mult));
+	    if (solid) {
+              XFillRectangle(display,win, gc,
+                  windows[windowno]->x + windows[windowno]->xpos-1,
+                  1+windows[windowno]->y + windows[windowno]->height -
+                  (points[count][point_pos] * windows[windowno]->height) /
+                  (graphs[count].scale * graphs[count].scale_mult),
+                  1,
+                  (points[count][point_pos] * windows[windowno]->height) /
+                  (graphs[count].scale * graphs[count].scale_mult));
+	    } else {
+		XDrawLine(display,win, gc,
+		    windows[windowno]->x + windows[windowno]->xpos-1,
+		    windows[windowno]->y + windows[windowno]->height-
+		    lasty[count] * windows[windowno]->height/(graphs[count].scale *
+		    graphs[count].scale_mult),
+		    windows[windowno]->x + windows[windowno]->xpos,
+		    windows[windowno]->y +windows[windowno]->height - 
+		    (points[count][point_pos] *windows[windowno]->height) /
+		    (graphs[count].scale * graphs[count].scale_mult));
+	    }
 	    lasty[count] = points[count][point_pos];
 
 	    /* point_pos %2 is to draw it every other point. */
@@ -513,7 +685,7 @@ void update_window(int windowno)
 		 */
 		while (sline*graphs[count].scale_lines < graphs[count].scale) {
 		    XDrawPoint(display, win, gc, windows[windowno]->x+ windows[windowno]->xpos -1,
-			windows[windowno]->height - (sline *
+			windows[windowno]->y + windows[windowno]->height - (sline *
 			graphs[count].scale_lines * windows[windowno]->height) /
 			graphs[count].scale + graph_spacing);
 		    sline+=slineinc;
@@ -536,11 +708,13 @@ void graph_loop()
 		    split_width);
 		graphs[i].running_avg = (graphs[i].scale+1)*split_width *
 			graphs[i].scale_mult;
-		if (graphs[i].host_offset != localhost || allnames) {
+		if (!hide_hosts && 
+		    graphs[i].host_offset != localhost || allnames) {
 			strcat(graphs[i].name,"@");
-			strcat(graphs[i].name,hosts[graphs[i].host_offset]);
+			strcat(graphs[i].name,hosts[graphs[i].host_offset].name);
 		}
 		graphs[i].name_len = strlen(graphs[i].name);
+		graphs[i].max_val=0;
 	}
 
 	/* Create a pixmap for scrolling just about the size of the
@@ -554,6 +728,7 @@ void graph_loop()
 
 	set_first_values();
 	sleep(sleep_time);
+
 	set_values();		/* this call sets the starting point */
 	sleep(sleep_time);
 	for (i=0; i<split_x * split_y; i++)
@@ -589,26 +764,33 @@ void usage()
 	puts("-baseline color		draw baseline of color 'color'");
 	puts("-border number		spacing between graphs and window edges");
 	puts("-color color		color of last graph specified by -type");
+	puts("-defcolor graphtype color	Sets default color for all types of graphtype");
 	puts("-display displayname	X server to contact");
 	puts("-foreground color	foreground color");
 	puts("-geometry geom		size (in pixels) and position");
 	puts("-hidelabels		do not display labels at bottom of graph.");
+	puts("-hidehosts		do not add hostnames to labels.");
 	puts("-host string		machine to report statistics on");
 	puts("-link graph		link scale of last graph to graph specified.");
 	puts("			For graphs on remote hosts, use graph@host from.");
 	puts("-min number		minimum scale of last graph specified by -type");
 	puts("-max number		maximum scale of last graph specified by -type");
+	puts("-name string		Set the X application resource name");
 	puts("-ruler number		draw a ruler below the baseline, with time markings");
 	puts("			every 'number' seconds.");
 	puts("-samescale		Synchronize scales of similar graphs");
 	puts("-sample number		how many seconds between each update");
 	puts("-scale number		draw dotted horizontal lines of the last graph");
 	puts("			at value specified.");
+	puts("-showmax			show running maximum value");
+	puts("-solid			Draw a solid graph.");
 	puts("-split WxH		Split the window into W width and H height sections.");
+	puts("-title string		Set the window manager title");
 	puts("-type string		type of graph to display: Choices of cpu,");
-	puts("			packets, page, swap, interrupts, disk, context, load,");
-	puts("			collisions, or errors.");
+	puts("			packets, pagei, swap, interrupts, disk, context, load,");
+	puts("			collisions, errors, ipackets, opackets, apagei, apageo, pageo.");
 	puts("-window number		Window number to plot graph in.");
+	puts("-wtitle name		Sets the default short subwindow name.");
 	
 	exit(1);
 }
@@ -630,7 +812,7 @@ void set_up_graph(char *graphtype)
 		*host='\0';
 		host++;
 	}
-	for (i=0; i <= NUM_TYPES; i++) {
+	for (i=0; i < NUM_TYPES; i++) {
 	    if (!strcmp(graphtype, types[i].cmd_option)) {
 		if (num_graphs==0)
 			graphs = (struct graph_info *)
@@ -642,7 +824,7 @@ void set_up_graph(char *graphtype)
 		graphs[num_graphs].type = i;
 		graphs[num_graphs].scale = types[i].def_scale;
 		graphs[num_graphs].true_scale = types[i].def_scale;
-		graphs[num_graphs].color_pixel = foreground.pixel;
+		graphs[num_graphs].color_pixel = defcolors[i];
 		strcpy(graphs[num_graphs].name,types[i].window_name);
 		graphs[num_graphs].host_offset = -1;
 		graphs[num_graphs].scale_lines = 0;
@@ -654,7 +836,7 @@ void set_up_graph(char *graphtype)
 			if (num_windows_spec > (split_x * split_y-1))
 				num_windows_spec--;
 		}
-		if (i==CPU) {
+		if (i==CPU || i==UCPU || i==NICECPU || i==SCPU || i==ICPU) {
 			graphs[num_graphs].max_scale = 100;
 			graphs[num_graphs].min_scale = 100;
 		}
@@ -670,7 +852,7 @@ void set_up_graph(char *graphtype)
 		break;
 	    }
 	}
-	if (i>NUM_TYPES) {
+	if (i>=NUM_TYPES) {
 		fprintf(stderr,"Unknown graph type: %s\n", graphtype);
 		exit(1);
 	}
@@ -759,7 +941,12 @@ void set_sample_time(char *val)
 
 void set_allnames() {	allnames=TRUE;	}
 void set_samescale() {	scale_sync = TRUE; }
-void set_hidelabels() {	hide_labels = TRUE; }
+void set_hidelabels() {	label_style = Nolabel; }
+void set_solid() {	solid=TRUE; }
+void set_hidehosts() { hide_hosts = TRUE; }
+void set_name(char *val) {     window_name = val; }
+void set_title(char *val) {     window_title = val; }
+void set_showmax() { show_max = TRUE; }
 
 void set_ruler(char *val)
 {
@@ -806,6 +993,7 @@ void set_split (char *split)
 	windows[tmpx] = malloc(sizeof(struct Xss_Window) * split_x * split_y);
 	windows[tmpx]->num_graphs = 0;
 	windows[tmpx]->redraw_needed = FALSE;
+	windows[tmpx]->title = NULL;
     }
 }
 
@@ -858,8 +1046,11 @@ void set_graph_host(char *host)
 	fatal("No graphs have been specified");
     else {
 	if (!num_hosts) {
-		hosts = (char **) malloc(sizeof(char *));
-		hosts[0] = strdup_local(host);
+		hosts = (struct Host_Info*) malloc(sizeof(struct Host_Info));
+		hosts[0].name = strdup_local(host);
+#ifdef USE_NEW_RSTAT
+		hosts[0].client=NULL;
+#endif
 		graphs[num_graphs-1].host_offset = 0;
 		num_hosts++;
 	}
@@ -867,10 +1058,14 @@ void set_graph_host(char *host)
 		int s;
 
 		for (s=0; s<num_hosts; ++s)
-			if (!strcmp(hosts[s],host)) break;
+			if (!strcmp(hosts[s].name,host)) break;
 		if (s==num_hosts) {
-			hosts =(char**)realloc(hosts, (num_hosts+1) * sizeof(char *));
-			hosts[num_hosts++] = strdup_local(host);
+		    hosts =(struct Host_Info *)realloc(hosts, (num_hosts+1) * sizeof(struct Host_Info));
+		    hosts[num_hosts].name = strdup_local(host);
+#ifdef USE_NEW_RSTAT
+		    hosts[num_hosts].client=NULL;
+#endif
+		    num_hosts++;
 		}
 		graphs[num_graphs-1].host_offset = s;
 	}
@@ -904,19 +1099,19 @@ void set_link(char *link)
 		*host='\0';
 		host++;
 	}
-	for (tmp=0; tmp<=NUM_TYPES; tmp++)
+	for (tmp=0; tmp<NUM_TYPES; tmp++)
 	    if (!strcmp(graph, types[tmp].cmd_option)) {
 		graph_type = tmp;
 		break;
 	    }
-	if (tmp>NUM_TYPES) {
+	if (tmp>=NUM_TYPES) {
 		fprintf(stderr,"Could not match type to %s\n",graph);
 		exit(1);
 	}
 	for (tmp=0; tmp<(num_graphs-1); tmp++)
 	     if (graphs[tmp].type == graph_type) {
 		if ((host==NULL && graphs[tmp].host_offset == -1) ||
-		    (graphs[tmp].host_offset!=-1 && !strcmp(host, hosts[graphs[tmp].host_offset])))
+		    (graphs[tmp].host_offset!=-1 && !strcmp(host, hosts[graphs[tmp].host_offset].name)))
 			break;
 	     }
 
@@ -934,49 +1129,97 @@ void set_link(char *link)
 }
 
 
+void set_window_title(char *val)
+{
+    int winnum=next_window;
+
+    if (winnum==-1) {
+	winnum=0;
+    }
+    if (winnum>split_x*split_y) {
+	fprintf(stderr,"Internal error - next_window is too high (%d)\n" , next_window);
+	return;
+    }
+    if (windows[winnum]->title)
+	free(windows[winnum]->title);
+    windows[winnum]->title = strdup_local(val);
+    label_style = Minimal;
+}
+	
+
 void set_window(char *val)
 {
     int tmp_window;
 
     tmp_window=atoi(val);
-    if (tmp_window<0 || tmp_window>split_x*split_y)
+    if (tmp_window<1 || tmp_window>split_x*split_y)
 	fatal("Invalid window number.");
 
     next_window=--tmp_window;
 }
 
+void set_def_graph_color(char *graphtype, char *color)
+{
+    int i;
+    XColor	exact, screen_def;
+
+    for (i=0; i < NUM_TYPES; i++) {
+	if (!strcmp(graphtype, types[i].cmd_option)) {
+
+	    if (!XAllocNamedColor(display, DefaultColormap(display, screen_num),
+				  color, &screen_def,&exact))
+		fprintf(stderr,"Could not allocated color %s\n",color);
+	    else {
+		defcolors[i]=screen_def.pixel;
+	    }
+	    break;
+	}
+    }
+    if (i>=NUM_TYPES) {
+	fprintf(stderr,"Unknown graph type: %s\n", graphtype);
+    }
+}
+
+
 /* This table is set up by pass number the options are processed.
  * As of now, this order is not used, but it makes things a bit clearer
  */
 struct Command_Line_Options options[]= {
-{"-display", TRUE, 1, set_display},	/* display needs to be set before */
+{"-display", 1, 1, set_display},	/* display needs to be set before */
 					/* anything else */
 
-{"-split", TRUE, 2, set_split},
-{"-background", TRUE, 2, set_background_color},
-{"-bg", TRUE, 2, set_background_color},
-{"-foreground", TRUE, 2, set_foreground_color},
-{"-fg", TRUE, 2, set_foreground_color},
-{"-baseline", TRUE, 2, set_baseline},
-{"-sample", TRUE, 2, set_sample_time},
-{"-allnames", FALSE, 2, set_allnames},
-{"-samescale", FALSE, 2, set_samescale},
-{"-hidelabels", FALSE, 2, set_hidelabels},
-{"-ruler", TRUE, 2, set_ruler},
-{"-border", TRUE, 2, set_border},
+{"-name", TRUE, 2, set_name},
+{"-title", TRUE, 2, set_title},
+{"-split", 1, 2, set_split},
+{"-background", 1, 2, set_background_color},
+{"-bg", 1, 2, set_background_color},
+{"-foreground", 1, 2, set_foreground_color},
+{"-fg", 1, 2, set_foreground_color},
+{"-baseline", 1, 2, set_baseline},
+{"-sample", 1, 2, set_sample_time},
+{"-allnames", 0, 2, set_allnames},
+{"-showmax", 0, 2, set_showmax},
+{"-samescale", 0, 2, set_samescale},
+{"-hidelabels", 0, 2, set_hidelabels},
+{"-ruler", 1, 2, set_ruler},
+{"-border", 1, 2, set_border},
+{"-defcolor",2, 2, set_def_graph_color},
+{"-hidehosts", 0, 2, set_hidehosts},
 
-{"-geometry", TRUE, 3, parse_geometry},		/* Geometry needs to be set */
+{"-geometry", 1, 3, parse_geometry},		/* Geometry needs to be set */
 						/* after split */
 
 /* Stage 4 is really the graphs and their options */
-{"-type", TRUE, 4, set_up_graph},
-{"-color", TRUE, 4, set_graph_color},
-{"-min", TRUE, 4, set_graph_min},
-{"-max", TRUE, 4, set_graph_max},
-{"-host", TRUE, 4, set_graph_host},
-{"-scale", TRUE, 4, set_graph_scale},
-{"-link", TRUE, 4, set_link},
-{"-window", TRUE, 4, set_window}
+{"-type", 1, 4, set_up_graph},
+{"-color", 1, 4, set_graph_color},
+{"-min", 1, 4, set_graph_min},
+{"-max", 1, 4, set_graph_max},
+{"-host", 1, 4, set_graph_host},
+{"-scale", 1, 4, set_graph_scale},
+{"-link", 1, 4, set_link},
+{"-window", 1, 4, set_window},
+{"-wtitle", 1, 4, set_window_title},
+{"-solid", 0, 4, set_solid}
 };
 
 void parse_args(int argc, char *argv[], int pass)
@@ -990,18 +1233,20 @@ void parse_args(int argc, char *argv[], int pass)
 		 * this pass.  Just skip over it
 		 */
 		if (options[i].pass != pass) {
-		    if (options[i].num_args) on_arg+=2;
-		    else on_arg++;
+		    on_arg += options[i].num_args+1;
 		    break;
 		}
 		if (options[i].num_args) {
-		    if ((on_arg+1)==argc) {
+		    if ((on_arg+options[i].num_args)==argc) {
 			fprintf(stderr,"%s requires an argument.\n", options[i].cmd_option);
 			exit(1);
 		    }
 		    else {
-			options[i].func(argv[on_arg+1]);
-			on_arg +=2;
+			if (options[i].num_args==1)
+				options[i].func(argv[on_arg+1]);
+			if (options[i].num_args==2)
+				options[i].func(argv[on_arg+1],argv[on_arg+2]);
+			on_arg +=options[i].num_args+1;
 		    }
 		}
 		else {
@@ -1031,7 +1276,7 @@ static void set_min_geometry()
 }
 
 
-void main( argc, argv )
+int main( argc, argv )
 	int             argc;
 	char           *argv[];
 
@@ -1039,16 +1284,27 @@ void main( argc, argv )
 	XGCValues	xgcvalues;
 	XWMHints	wm_hints;
 	XClassHint	class_hints;
-	XTextProperty	window_name;
+	XTextProperty	wm_title;
 	XSetWindowAttributes	window_attributes;
 	int		i;
-	char		*prog_name=VERSION;
+	char		*cp;
 
 	/* localhost needs to be static because we set a pointer to it. */
 	static char	localhostname[MAXHOSTNAMELEN];
 
 	if (argc==1) usage();
 
+	/* By convention, the name of the executable is used as the default
+	 * application name.
+	 */
+	for ( window_name = argv[0] + strlen(argv[0]); window_name > argv[0];
+	     --window_name )
+	    if (*window_name == '/') {
+		 ++window_name;
+		 break;
+	    }
+
+
 	/* Find the display (and screen number) first.  Once we have that
 	 * information, we can process the other arguements better as
 	 * we read them.
@@ -1069,11 +1325,14 @@ void main( argc, argv )
 	foreground.red = foreground.blue = foreground.green = 0;
 	background.pixel = whitepixel;
 	background.red = foreground.blue = foreground.green = 65535;
+	for (i=0; i<NUM_TYPES; i++)
+		defcolors[i]=foreground.pixel;
 
 	windows = malloc(sizeof(struct Xss_Window*));
 	windows[0]= malloc(sizeof(struct Xss_Window) * split_x * split_y);
 	windows[0]->num_graphs = 0;
 	windows[0]->redraw_needed = FALSE;
+	windows[0]->title=NULL;
 
 	parse_args(argc, argv,2);
 	parse_args(argc, argv,3);
@@ -1096,7 +1355,7 @@ void main( argc, argv )
 	set_min_geometry();
 
 	for (i=0; i<num_hosts; i++) {
-		if (!strcmp(hosts[i],localhostname)) {
+		if (!strcmp(hosts[i].name,localhostname)) {
 			localhost = i;
 			break;
 		}
@@ -1117,15 +1376,18 @@ void main( argc, argv )
 		windows[graphs[i].window]->num_graphs++;
 
 		if (graphs[i].host_offset == -1) {
-			if (localhost==-1) {
-				if (num_hosts==0)
-					hosts = (char **)malloc(sizeof(char *));
-				else 
-					hosts=(char**)realloc(hosts, (1+num_hosts) * sizeof(char *));
-				hosts[num_hosts] = localhostname;
-				localhost = num_hosts++;
-			}
-			graphs[i].host_offset = localhost;
+		    if (localhost==-1) {
+			if (num_hosts==0)
+			    hosts = (struct Host_Info *)malloc(sizeof(struct Host_Info));
+			else 
+			    hosts=(struct Host_Info *)realloc(hosts, (1+num_hosts) * sizeof(struct Host_Info));
+			hosts[num_hosts].name = localhostname;
+#ifdef USE_NEW_RSTAT
+			hosts[num_hosts].client = NULL;
+#endif
+			localhost = num_hosts++;
+		    }
+		    graphs[i].host_offset = localhost;
 		}
 	}
 
@@ -1149,7 +1411,8 @@ void main( argc, argv )
 
 	xgcvalues.foreground = foreground.pixel;
 	xgcvalues.background = background.pixel;
-	xgcvalues.line_width = 1;
+/*	xgcvalues.line_width = 1;*/
+	xgcvalues.line_width = 0;
 	
 	xgcvalues.graphics_exposures = False;
 	xgcvalues.font = font_info->fid;
@@ -1163,13 +1426,13 @@ void main( argc, argv )
 		GCForeground | GCBackground | GCGraphicsExposures, &xgcvalues);
 
 	wm_hints.flags = InputHint;
-	wm_hints.input = False;
+	wm_hints.input = True;
 
-	class_hints.res_name = "XSysStats";
+	class_hints.res_name = window_name;
 	class_hints.res_class =  "XSysStats";
 
-	XStringListToTextProperty(&prog_name, 1, &window_name);
-	XSetWMProperties(display, win, &window_name, &window_name,
+	XStringListToTextProperty(&window_title, 1, &wm_title);
+	XSetWMProperties(display, win, &wm_title, &wm_title,
 		argv,argc, &size_hints, &wm_hints, &class_hints);
 
 
@@ -1178,6 +1441,7 @@ void main( argc, argv )
 
 	/* initialize random (used to determine next stage) */
 	graph_loop();
+	exit(0);
 
 }
 
